@@ -30,15 +30,21 @@ import pytest
 
 from vdsm import constants
 from vdsm.storage import blockSD
+from vdsm.storage import clusterlock
 from vdsm.storage import constants as sc
 from vdsm.storage import exception as se
 from vdsm.storage import lvm
+from vdsm.storage import resourceManager as rm
 from vdsm.storage import sd
+from vdsm.storage import sp
 from vdsm.storage.sdc import sdCache
+from vdsm.storage.spbackends import StoragePoolDiskBackend
 
 from . import qemuio
 from . marks import requires_root, xfail_python3
 from storage.storagefakelib import fake_vg
+from storage.storagefakelib import FakeDomainMonitor
+from storage.storagefakelib import FakeTaskManager
 
 TESTDIR = os.path.dirname(__file__)
 
@@ -364,6 +370,8 @@ def test_volume_life_cycle(monkeypatch, tmp_storage, tmp_repo, fake_access,
     # Attache repo pool - SD expects at least one pool is attached.
     dom.attach(tmp_repo.pool_id)
 
+    lvm.set_read_only(False)
+
     with monkeypatch.context() as mc:
         mc.setattr(time, "time", lambda: 1550522547)
         dom.createVolume(
@@ -488,6 +496,8 @@ def test_volume_metadata(tmp_storage, tmp_repo, fake_access, fake_rescan,
     img_uuid = str(uuid.uuid4())
     vol_uuid = str(uuid.uuid4())
 
+    lvm.set_read_only(False)
+
     dom.createVolume(
         desc="old description",
         diskType="DATA",
@@ -577,8 +587,9 @@ def test_create_snapshot_size(
     vol_uuid = str(uuid.uuid4())
     vol_capacity = 2 * parent_vol_capacity
 
-    # Create parent volume.
+    lvm.set_read_only(False)
 
+    # Create parent volume.
     dom.createVolume(
         imgUUID=img_uuid,
         capacity=parent_vol_capacity,
@@ -652,3 +663,59 @@ def test_create_snapshot_size(
 
     actual = parent_vol.getInfo()
     assert int(actual["capacity"]) == parent_vol_capacity
+
+
+@requires_root
+@pytest.mark.root
+def test_spm_lifecycle(
+        tmp_storage,
+        tmp_repo,
+        fake_access,
+        fake_rescan,
+        tmp_db,
+        fake_task,
+        fake_sanlock):
+
+    msd_uuid = str(uuid.uuid4())
+
+    dev = tmp_storage.create_device(20 * 1024 ** 3)
+    lvm.createVG(msd_uuid, [dev], blockSD.STORAGE_UNREADY_DOMAIN_TAG, 128)
+    vg = lvm.getVG(msd_uuid)
+
+    master_dom = blockSD.BlockStorageDomain.create(
+        sdUUID=msd_uuid,
+        domainName="domain",
+        domClass=sd.DATA_DOMAIN,
+        vgUUID=vg.uuid,
+        version=5,
+        storageType=sd.ISCSI_DOMAIN)
+
+    sdCache.knownSDs[msd_uuid] = blockSD.findDomain
+    sdCache.manuallyAddDomain(master_dom)
+
+    # TODO: move to tmp_repo and check if can replace fake resource manager.
+    rm.registerNamespace(sc.STORAGE, rm.SimpleResourceFactory())
+
+    pool = sp.StoragePool(
+        tmp_repo.pool_id,
+        FakeDomainMonitor(),
+        FakeTaskManager())
+    pool.setBackend(StoragePoolDiskBackend(pool))
+    pool.create(
+        poolName="pool",
+        msdUUID=msd_uuid,
+        domList=[msd_uuid],
+        masterVersion=0,
+        leaseParams=sd.DEFAULT_LEASE_PARAMS)
+
+    # By default a host is in a read-only lvm mode.
+    assert lvm.is_read_only()
+
+    pool.startSpm(prevID=0, prevLVER=0, maxHostID=clusterlock.MAX_HOST_ID)
+    try:
+        # SPM host is in a read/write lvm mode.
+        assert not lvm.is_read_only()
+    finally:
+        pool.stopSpm()
+    # non-SPM host is in a read-only lvm mode.
+    assert lvm.is_read_only()
